#ifndef FIT_TOWER_H
#define FIT_TOWER_H

#include <iostream>
#include  "TObject.h"
#include  "CalibStr.h"
#include "Geom.h"
#include "TMatrix.h"
#include "TMinuit.h"
#include "WasExternal.h"
#include "TF2.h"
#include "TowerUtil.h"
#include "TMath.h"
#include "TowerFPD.h"

class FitTower : public TObject
{
 public:
  
  TMinuit*    fMn;    // Minuit fitter ma
  FitTower(TMatrix* ptm, Geom* pgeom,Int_t iew,Int_t nstb);
  FitTower(const Int_t dim[2], const Double_t wd, TF2 *ssFunct);
  ~FitTower();
  TF2* fcnSS;
  TObjArray* tow2Fit;

  static  WasExternal we;
  Double_t step[3*MAX_NUMB_PHOTONS+1];
  TowerUtil* pTowerUtil;
  static Double_t GGams(Double_t *x, Double_t *par);
  static Double_t FGams(Double_t *x, Double_t *par);
  
  static void  Fcn1(Int_t & npar, Double_t *grad,  
		    Double_t &fval, Double_t *par, Int_t iflag);
  static void Fcn2(Int_t & nparam, Double_t *grad, 
		   Double_t &fval, Double_t *param, Int_t iflag);
  void SetRow(const Int_t row) {fRow=row;};
  void SetCol(const Int_t col) {fCol=col;};
  void SetTWidthCM(const Double_t tw) {fTWidthCM = tw;};
  void SetFCN(void (*fcn)(Int_t &, Double_t *, Double_t &, Double_t *, Int_t));
  void SetFunctShowShape(TF2 *fSS) {fFunctSS = fSS;};
  void SetNumberPhoton(const Int_t nP);

  void GetRow(Int_t &row) {row=fRow;};
  void GetCol(Int_t &col) {col=fCol;};
  void GetParameter(Double_t *ap){/* not implemeted*/};
  void GetTWidthCM(Double_t &tw) {tw=fTWidthCM;};
  TF2* GetFunctShowShape(void) {return fFunctSS;};
  Int_t Fit(const Double_t *par, const Double_t *step, 
	    const Double_t *low, const Double_t *up);
  Int_t Fit2Pin1Clust(const Double_t *para, const Double_t *step, 
		      const Double_t *low, const Double_t *up);
 private:	
  
  ClassDef(FitTower,3);
  void SetStep();
  
  Int_t       fCol;           // number of columns for lead glass array
  Int_t       fRow;           // number of rows for lead glass array
  Double_t    fTWidthCM;      // width of one lead glass module
  TF2 *       fFunctSS;       // shower-shape function
  Double_t    fNumbPhotons;   // number of photons to be fitted: Minuit wants a Double_t!
  Double_t    fFitPara[3*MAX_NUMB_PHOTONS+1];  // Minuit fit parameter

};

#endif
