
#include <iostream>
#include <stdlib.h>
#include <math.h>

#include "TMath.h"
#include "TVector2.h"

#include "TowerFPD.h"
#include "PhotonHitFPD.h"
#include "HitCluster.h"


ClassImp(HitCluster);




//HitCluster *clust2Fit;


HitCluster::HitCluster()
{

  Clear();
  
  
}


HitCluster::~HitCluster()
{
  
}


void HitCluster::Clear(void)
{
  catag   = -1 ;
  numbTower = nPhoton =  0 ;
  energy  =  0 ;
  x0 = y0 = sigmaX = sigmaY = sigmaXY = chiSquare = sigmaMin = sigmaMax = -1 ;
  thetaAxis = -10 ;
  for(Int_t ip=0; ip<MAX_PHOTON_PER_CLUSTER; ip++) {
    photon[ip].Clear();
  }
}


void HitCluster::Print(void)
{
  
  for(Int_t i=0; i<tow->GetEntriesFast(); i++) {
    ( (TowerFPD *) tow->At(i) )->Print();
  }
  
  cout << "catag=" << catag << "\t numbTower=" << numbTower << "\t nPhoton=" << nPhoton << "\t energy=" << energy ;
  cout << "\t (" << x0 << ", " << y0 << ")\t (" << sigmaX << ", " << sigmaY << ", " << sigmaXY << ")" << endl;

	cout << "thetaAxis=" << thetaAxis << ",\t sigmaMin=" << sigmaMin << ",\t sigmaMax=" << sigmaMax << endl;

	for(Int_t j=0; j<nPhoton; j++) {
	  cout << "Photon #" << j << ":\n";
		photon[j].Print();
	}
	cout << "chiSquare = " << chiSquare << "\n" << endl;

}


// 2003-10-13
// Use matrix algebra to find the axis. Basically, the 2nd moments form a symmetrical 2-D matrix. Diagonize it
//   and the two eigen-values are maximum moment and minimum moment, while two eigen-vectors are the vectors of
//   axis. To speed up the process, the solution is explicitly written in "C" (instead of using TMatrix).
// 
//
// do a 1-parameter fix (theta angle in x-y plane) to find the "least-2nd-sigma" axis of the cluster
//
void HitCluster::FindClusterAxis(void)
{
	Double_t dSigma2, aA, bB;
	dSigma2 = sigmaX * sigmaX - sigmaY * sigmaY ;
	aA = sqrt( dSigma2 * dSigma2 + 4.0 * sigmaXY * sigmaXY ) + dSigma2 ;
	bB = 2 * sigmaXY ;

	if( sigmaXY < 1e-10 ) {
		if( aA < 1e-10 ) {
			bB = sqrt( dSigma2 * dSigma2 + 4.0 * sigmaXY * sigmaXY ) - dSigma2 ;
			aA = 2 * sigmaXY ;
		}
	}


	thetaAxis = atan2(bB, aA);


	Double_t myPi = TMath::Pi() ; 
	while( thetaAxis > (myPi / 2.0) )
		thetaAxis -= myPi;

	while( thetaAxis < -(myPi / 2.0) )
		thetaAxis += myPi;

	sigmaMin = sqrt( GetSigma(thetaAxis) / energy ) ;
	sigmaMax = sqrt( GetSigma(thetaAxis - TMath::Pi() / 2.0) / energy ) ;

}



// calculate sigma w.r.t the axis going through the "center" and of an angle "theta" in x-y plane
//
Double_t HitCluster::GetSigma(Double_t theta){

	Double_t sigma = 0 ;

	// 2-d vector vaxis define the axis
	//
	TVector2 vaxis( cos(theta), sin(theta) );


	// loop over all towers pointer in cluster
	//
	TowerFPD * oneTower;
	for(Int_t it=0; it<tow->GetEntriesFast(); it++) {

		oneTower = (TowerFPD *) tow->At(it);
		// the 2-d vector from the "center" of cluster to tower
		// "center" are at 0.5, 1.5, etc! Need shift of 0.5
		//
		TVector2 v1( oneTower->col - 0.5 - x0, oneTower->row - 0.5 - y0 );
		
		// perpendicular distance to the axis = length of the component of vector "v1" that is norm to "vaxis"
		//
		Double_t dis = (v1.Norm(vaxis)).Mod();
		
		// contribution to sigma
		//
		sigma += oneTower->energy * dis * dis ;
	}
	
	return sigma;
}


