// ROOT
#include "TMatrix.h"
// STAR
#include "StEventTypes.h"
#include "StMuDSTMaker/COMMON/StMuTypes.hh"

// PSU-FMS package
#include "FilesSet.h"
#include "Geom.h"
#include "Qt.h"
#include "CalibStr.h"
// Local
#include "StFmsA2EMaker.h"
#include "StFmsTriggerDetectorAccess.h"

#include <iostream>
#include <fstream>
using namespace std;

ClassImp(StFmsA2EMaker);
unsigned short StFmsA2EMaker::adc(int nstb,int row, int col)
{
  //error returns 0
  if(nstb<1 || nstb>4)return 0;
  if(row<1 || row>mQt->ROW_NUM[nstb-1])return 0;
  if(col<1 || col>mQt->COL_NUM[nstb-1])return 0;
  return (unsigned short) (*((TMatrix*) (mQt->qtMat->At(nstb-1))))(row-1,col-1);
};
float StFmsA2EMaker::energy(int nstb,int row, int col)
{
  float ped=Rped->GetValue(2,nstb,row-1,col-1);
  float gain=Rgain->GetValue(2,nstb,row-1,col-1);
  float gaincorr=Rgaincorr->GetValue(2,nstb,row-1,col-1);
  return (adc(nstb,row,col)-ped)*gain*gaincorr;
};

StFmsA2EMaker::~StFmsA2EMaker()
{
  if(mFiles)delete mFiles;
  if(mGeom)delete mGeom;
  if(mQt)delete mQt;
  if(Rped)delete Rped;
  if(Rgain)delete Rgain;
  if(Rgaincorr)delete Rgaincorr;

}

void StFmsA2EMaker::Clear(Option_t* option)
{
  StMaker::Clear(option);
}

void StFmsA2EMaker::LinkFiles(FilesSet* fls)
{
  mFiles=fls;
};
int StFmsA2EMaker::Init()
{

  // Read in pedestal, gains, calibration, etc. files
  //  via PSU package

  if(mFiles==0)
    {
      cout<<"Must Link to a FilesSet before Init \n";
      return 0;
    };
  mGeom = new Geom(mFiles);
  mQt=new Qt(mFiles);
  mQt->EnableqtHist();
  currentRunNumber= 9067059;  // Some number from run 8
  
  Rped=new CalibStr(currentRunNumber,mFiles->p_fpdped()->Path());
  Rgain=new CalibStr(currentRunNumber,mFiles->p_fpdgain()->Path());
  Rgaincorr=new CalibStr(currentRunNumber,mFiles->p_fpdgaincorr()->Path());

  return StMaker::Init();
}

int StFmsA2EMaker::Make()
{
  // Get MuDst dataset. If the call to GetDataSet() returns 0,
  // then no MuDst was found. At which point, issue a warning
  // and exit Make().
  
  if (!GetDataSet("MuDst")) 
    {
      LOG_WARN << "No MuDst" << endm;
      return kStWarn;
    }
  
  UInt_t nhits=((StFmsTriggerDetectorAccess) 
		(StMuDst::event()->fmsTriggerDetector())).GetmNumQTdata();
  
  UInt_t* QTdata=((StFmsTriggerDetectorAccess) 
                  (StMuDst::event()->fmsTriggerDetector())).GetmQTdata();
  mQt->decodeQT(nhits,QTdata,0);
  

  return kStOk;
}

int StFmsA2EMaker::Finish()
{


  return kStOk;
}
